## SETTING UP ENVIRONMENT
# Input
library(Sequgio)
library(GenomicFeatures);

# Loading example data
library(RNAseqData.HNRNPC.bam.chr14)

# Example TXDB object
library(TxDb.Hsapiens.UCSC.hg19.knownGene)

# Setting parameters for parallel computing
param_core <- MulticoreParam(8);


## CREATING ANNOTATION TEMPLATE (TXDB)
# USE THE EXAMPLE ONE (loading in TxDb.Hsapiens.USCS.hg19.knownGene)
# OR
# UNCOMMENT below to create your own based on the .gtf file
# NOTE: change the directory to point your .gtf file
# Txdb.igenomes_UCSC_hg19_2016_06 <- makeTranscriptDbFromGFF(file="/path/to/genes.gtf",
#                format="gtf",
#                dataSource=paste("http://cufflinks.cbcb.umd.edu/igenomes.html", sep=""),
#                species="hg19.2014-06-02-13-47-56")
# Saving to a file
# saveDb(Txdb.igenomes_UCSC_hg19_2016_06,file="Txdb.igenomes_UCSC_hg19_2016_06.sqlite")

# Limiting annotation template to chromosome 14 for speeding up example calculations
seqs = seqnames(seqinfo(Txdb.igenomes_UCSC_hg19_2016_06))
sel =  rep(FALSE,length(seqs))
names(sel) =  seqs
sel["chr14"] = TRUE
isActiveSeq(Txdb.igenomes_UCSC_hg19_2016_06) =  sel
txdbrs = reshapeTxDb(Txdb.igenomes_UCSC_hg19_2016_06,probelen = 48L,mcpar=param_core)

# UNCOMMENT below to include the whole genome
# seqs = seqnames(seqinfo(Txdb.igenomes_UCSC_hg19_2016_06))
# sel =  rep(TRUE,length(seqs))
# sel["chr14"] = TRUE
# isActiveSeq(Txdb.igenomes_UCSC_hg19_2016_06) =  sel
# txdbrs = reshapeTxDb(Txdb.igenomes_UCSC_hg19_2016_06,probelen = 48L,mcpar=param_core)


# Exporting txdb to file in table format
write.table(as.data.frame(txdbrs), "Txdb.igenomes_UCSC_hg19_2016_06.txt", sep="\t", row.names=FALSE, col.names=FALSE, quote = FALSE)

# Exporting exon names to file
ex_list <- split(values(txdbrs@unlistData)$exon_name,values(txdbrs@unlistData)$tx_name)
reg_vec <- sapply(split(values(txdbrs@unlistData)$region_id,values(txdbrs@unlistData)$tx_name),function(x) x[1])
sizes_ex_list <- sapply(ex_list,length)
n.exons <- sum((sizes_ex_list^2+sizes_ex_list)/2)
exons.names <- unique(.Call("makeExNames",ex_list,reg_vec,as.integer(n.exons)))
write.table(exons.names, "exons_Txdb.igenomes_UCSC_hg19_2016_06.txt", row.names=FALSE, col.names=FALSE, quote=FALSE, sep="\t")

## CREATING DESING MATRIX
Design <- makeXmatrix(txdbrs,method="PE",mulen=138,sd=52,mcpar=param_core)

## IMPORTING BAM FILES AND CREATING COUNT MATRIX
# Importing BAM files
bflist <- BamFileList(RNAseqData.HNRNPC.bam.chr14_BAMFILES, asMates=TRUE)
seqObj <- setCounts(bflist,txdb,fileName="test")
doCounts(seqObj,mcpar=param_core)

## FITTING MODEL
iGenes = names(Design)
fitAll = NULL;
cat(length(iGenes),"\n");

for (i in 1:length(iGenes))
{
  tryCatch({
    cat("\n processing gene ",i);
    aFit <- fitModels(iGenes[i],design=Design,counts=allCounts)
    fitAll[[i]]=aFit}, error=function(e){cat("ERROR at gene ",i, " : ",conditionMessage(e), "\n")}
  )
}  
# exporting to file in table format
mydata=c()  
for (i in 1:length(fitAll))
{
  aFit=fitAll[[i]];      
  if (!is.null(aFit)){
    rownames(aFit) <- paste(i,"_",rownames(aFit),sep="");
    mydata <- rbind(mydata, aFit);          
  }
}

write.table(mydata, "isoformExpression.txt", quote=FALSE, sep = "\t"); 